@@ -573,7 +573,7 @@ ipv6_checkaddrflags(void *arg)
 	int ifa_flags;
 
 	ap = arg;
-	ifa_flags = if_addrflags6(ap->iface->name, &ap->addr);
+	ifa_flags = if_addrflags6(&ap->addr, ap->iface);
 	if (ifa_flags == -1)
 		syslog(LOG_ERR, "%s: if_addrflags6: %m", ap->iface->name);
 	else if (!(ifa_flags & IN6_IFF_TENTATIVE)) {
